@@ -24,11 +24,16 @@
 #include "DesktopJob.h"
 #include "FrameJob.h"
 
+#include <com/sun/star/beans/XPropertySet.hpp>
 #include <com/sun/star/lang/XMultiServiceFactory.hpp>
+#include <com/sun/star/lang/EventObject.hpp>
 #include <com/sun/star/frame/XFrame.hpp>
 #include <com/sun/star/frame/FrameSearchFlag.hpp>
 #include <com/sun/star/frame/XFramesSupplier.hpp>
+#include <com/sun/star/frame/XFrameActionListener.hpp>
 #include <com/sun/star/frame/XModel.hpp>
+#include <com/sun/star/frame/XLayoutManager.hpp>
+
 
 #include <gio/gio.h>
 #include <libdbusmenu-glib/server.h>
@@ -38,14 +43,18 @@
 #include <cppuhelper/implbase1.hxx>
 
 using rtl::OUString;
+using com::sun::star::beans::XPropertySet;
 using com::sun::star::beans::NamedValue;
 using com::sun::star::frame::XFrame;
 using com::sun::star::frame::XFramesSupplier;
 using com::sun::star::frame::XFrameActionListener;
 using com::sun::star::frame::XModel;
+using com::sun::star::frame::XLayoutManager;
 using com::sun::star::frame::FrameActionEvent;
+using com::sun::star::frame::XFrameActionListener;
 using com::sun::star::lang::IllegalArgumentException;
 using com::sun::star::lang::XMultiServiceFactory;
+using com::sun::star::lang::EventObject;
 using com::sun::star::uno::Any;
 using com::sun::star::uno::Exception;
 using com::sun::star::uno::Reference;
@@ -55,6 +64,33 @@ using com::sun::star::uno::UNO_QUERY;
 using com::sun::star::uno::UNO_QUERY_THROW;
 using com::sun::star::uno::XInterface;
 
+/*
+class DesktopFrameHelper : public cppu::WeakImplHelper1 < XFrameActionListener >
+{
+  public:
+	DesktopFrameHelper()
+	{
+	}
+
+	virtual ~DesktopFrameHelper()
+	{
+	}
+	
+	virtual void SAL_CALL frameAction(const FrameActionEvent& action)
+	  throw (RuntimeException)
+	{
+		g_debug ("frame action 00--");
+	}
+	
+	virtual void SAL_CALL disposing(const EventObject& aEvent)
+	  throw (RuntimeException)
+	{
+		g_debug ("frame disposed");
+	}
+};
+
+*/
+
 
 //--- DBus Callbacks ---//
 static void
@@ -80,37 +116,45 @@ Any SAL_CALL DesktopJob::execute( const Sequence< NamedValue >& aArguments )
 	g_type_init ();
 	
 	g_bus_own_name(G_BUS_TYPE_SESSION,
-               "org.libreoffice.dbusmenu",
+               LIBREOFFICE_BUSNAME,
                G_BUS_NAME_OWNER_FLAGS_NONE,
                on_bus,
                NULL,
                name_lost,
                NULL,
                NULL);
-
-	return Any();
 	
 	//Intent to export the root window
-/*
-	Reference < XFramesSupplier > xDesktop(mxMSF->createInstance(OUString ( RTL_CONSTASCII_USTRINGPARAM ("com.sun.star.frame.Desktop"))),
+	//TODO: Get the Window events right for this, try to not use Jobs.cxu for FrameJob
+
+	Reference < XFrame > xDesktop(mxMSF->createInstance(OUString ( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.frame.Desktop"))),
 	                                       UNO_QUERY);
-	if (!xDesktop.is())
+
+   	Reference< XPropertySet > frameProps (xDesktop, UNO_QUERY);
+
+
+	Sequence < com::sun::star::beans::Property > props = frameProps->getPropertySetInfo ()->getProperties ();
+	
+	/*for (int i = 0; i<props.getLength(); i++)
 	{
-		g_debug ("no desktop");
-		return Any();
+		g_debug ("%s", OUStringToOString (props[i].Name, RTL_TEXTENCODING_ASCII_US).getStr ());
 	}
+	OUString name;
+	name <<= frameProps->getPropertyValue (OUString::createFromAscii("Name"));
+		g_debug ("%s", OUStringToOString (name, RTL_TEXTENCODING_ASCII_US).getStr ());
+
+	Desktop->addFrameActionListener (new DesktopFrameHelper ());
+	
 	
 	Sequence < Reference < XFrame > > xFrames(xDesktop->getFrames()->queryFrames(com::sun::star::frame::FrameSearchFlag::ALL));
 		
 	for (gint32 i = 0; i < xFrames.getLength (); i++)
 	{
 		Reference < XFrame > xFrame (xFrames[i], UNO_QUERY);
-		FrameJob job (mxMSF);
-		job.exportMenus (xFrame);
-	}
+		xFrame->addFrameActionListener (new DesktopFrameHelper ());
+	}*/
 	
-	
-	return Any ();*/
+	return Any ();
 }
 
 OUString
