@@ -23,6 +23,7 @@
 
 #include "FrameHelper.h"
 
+#include <com/sun/star/awt/KeyEvent.hpp>
 #include <com/sun/star/awt/SystemDependentXWindow.hpp>
 #include <com/sun/star/awt/XSystemDependentWindowPeer.hpp>
 #include <com/sun/star/awt/XWindow2.hpp>
@@ -30,19 +31,27 @@
 #include <com/sun/star/container/XNameAccess.hpp>
 #include <com/sun/star/awt/XPopupMenu.hpp>
 #include <com/sun/star/awt/XMenuExtended.hpp>
+#include <com/sun/star/awt/XPopupMenuExtended.hpp>
+#include <com/sun/star/frame/XController.hpp>
 #include <com/sun/star/frame/XDispatch.hpp>
 #include <com/sun/star/frame/XDispatchHelper.hpp>
 #include <com/sun/star/frame/XDispatchProvider.hpp>
 #include <com/sun/star/frame/XLayoutManager.hpp>
+#include <com/sun/star/frame/XModel.hpp>
 #include <com/sun/star/frame/XModuleManager.hpp>
 #include <com/sun/star/frame/XPopupMenuController.hpp>
+#include <com/sun/star/frame/FrameAction.hpp>
+#include <com/sun/star/frame/FrameActionEvent.hpp>
 #include <com/sun/star/frame/status/Visibility.hpp>
 #include <com/sun/star/lang/SystemDependent.hpp>
 #include <com/sun/star/beans/PropertyValue.hpp>
 #include <com/sun/star/util/URL.hpp>
 #include <com/sun/star/util/XURLTransformer.hpp>
 #include <com/sun/star/ui/XUIElement.hpp>
-
+#include <com/sun/star/ui/XUIConfigurationManager.hpp>
+#include <com/sun/star/ui/XUIConfigurationManagerSupplier.hpp>
+#include <com/sun/star/ui/XAcceleratorConfiguration.hpp>
+#include <com/sun/star/ui/XModuleUIConfigurationManagerSupplier.hpp>
 #include <rtl/process.h>
 
 #include <gio/gio.h>
@@ -52,18 +61,24 @@ using rtl::OUString;
 using rtl::OString;
 using rtl::OUStringToOString;
 
+using namespace ::com::sun::star;
+
+using com::sun::star::awt::KeyEvent;
 using com::sun::star::awt::SystemDependentXWindow;
 using com::sun::star::awt::XMenuExtended;
 using com::sun::star::awt::XPopupMenu;
+using com::sun::star::awt::XPopupMenuExtended;
 using com::sun::star::awt::XSystemDependentWindowPeer;
 using com::sun::star::awt::XWindow2;
 using com::sun::star::beans::XPropertySet;
 using com::sun::star::beans::PropertyValue;
 using com::sun::star::container::XNameAccess;
 using com::sun::star::container::NoSuchElementException;
+using com::sun::star::frame::XController;
 using com::sun::star::frame::XDispatch;
 using com::sun::star::frame::XDispatchProvider;
 using com::sun::star::frame::XDispatchHelper;
+using com::sun::star::frame::XModel;
 using com::sun::star::frame::XModuleManager;
 using com::sun::star::frame::XLayoutManager;
 using com::sun::star::frame::XPopupMenuController;
@@ -75,12 +90,74 @@ using com::sun::star::uno::UNO_QUERY_THROW;
 using com::sun::star::uno::Sequence;
 using com::sun::star::uno::XComponentContext;
 using com::sun::star::ui::XUIElement;
+using com::sun::star::ui::XUIConfigurationManager;
+using com::sun::star::ui::XUIConfigurationManagerSupplier;
+using com::sun::star::ui::XAcceleratorConfiguration;
+using com::sun::star::ui::XModuleUIConfigurationManagerSupplier;
 using com::sun::star::util::URL;
 using com::sun::star::util::XURLTransformer;
 
 // Item callbacks
-void     item_activated     (DbusmenuMenuitem *item, guint timestamp, gpointer user_data);
-gboolean item_about_to_show (DbusmenuMenuitem *item, gpointer user_data);
+OString  xid_to_object_path (unsigned long xid);
+
+// Item activated. It distpatches the command associated to a given menu item.
+void
+item_activated (DbusmenuMenuitem *item, guint timestamp, gpointer user_data)
+{
+	FrameHelper *helper =  (FrameHelper*)user_data;
+	OUString command = OUString::createFromAscii(dbusmenu_menuitem_property_get (item, "CommandURL"));
+	helper->dispatchCommand (command);
+}
+
+// Rebuilds the submenu
+gboolean
+item_about_to_show (DbusmenuMenuitem *item, gpointer user_data)
+{
+	//Get the XMenu interface for the MenuBar UIElement                               
+	FrameHelper *helper = (FrameHelper*)user_data;
+	Reference < XFrame > xFrame  = helper->getFrame ();
+	Reference< XPropertySet > frameProps (xFrame, UNO_QUERY);
+	Reference < XLayoutManager > xLayoutManager(frameProps->getPropertyValue(OUString::createFromAscii("LayoutManager")),
+	                                            UNO_QUERY);
+	Reference < XUIElement > menuBar(xLayoutManager->getElement (OUString::createFromAscii("private:resource/menubar/menubar")),
+	                                 UNO_QUERY);
+	Reference < XPropertySet > menuPropSet (menuBar, UNO_QUERY);
+	
+	Reference < XMenu > xMenu(menuPropSet->getPropertyValue(OUString::createFromAscii("XMenuBar")),
+	                          UNO_QUERY);
+	
+	//Find xMenu for the first level item
+	Reference < XMenu > xSubMenu;
+	Reference < XMenuExtended > xMenuEx (xMenu, UNO_QUERY);
+	guint16 root_count = xMenu->getItemCount();
+	for (guint16 i = 0; i<root_count ;i++)
+	{
+
+		guint16 id = xMenu->getItemId (i);
+		if (id == 0)
+			continue;
+
+		OUString command = xMenuEx->getCommand (id);
+
+		//We must find the element with the same command URL
+		if (! OUString::createFromAscii (dbusmenu_menuitem_property_get (item, "CommandURL")).equals (command))
+			continue;
+
+		Reference <XPopupMenu> subPopup (xMenu->getPopupMenu (id), UNO_QUERY);
+		xSubMenu = Reference <XMenu> (subPopup, UNO_QUERY);
+		break;
+	}
+
+	//We only do this for toplevel items
+	if (xSubMenu.is ())
+	{
+		helper->rebuildMenu (xSubMenu, item);
+		return FALSE;
+	}
+	
+	//If it is not a toplevel item we stop trying to rebuild
+	return TRUE;
+}
 
 void
 destroy_menuitem (gpointer data)
@@ -210,7 +287,7 @@ class MenuItemStatusListener : public cppu::WeakImplHelper1 < XStatusListener >
 		gchar* c_url = g_utf16_to_utf8 (url.getStr(),
 			                            url.getLength(),
 			                            NULL, NULL, NULL);
-//		g_debug ("UPDATE %s", c_url);
+		//g_debug ("UPDATE %s", c_url);
 		
 		GHashTable *commandsInfo = helper->getCommandsInfo ();
 		MenuItemInfo *info = (MenuItemInfo*)g_hash_table_lookup (commandsInfo, (gpointer)c_url);
@@ -237,12 +314,16 @@ class MenuItemStatusListener : public cppu::WeakImplHelper1 < XStatusListener >
 			g_free (c_url);
 		}
 		
-		/* For some reason, URLs can slip through as labels? */
+		// For some reason, URLs can slip through as labels?
 		info->setEnabled ((gboolean)Event.IsEnabled);
 		
 		if ((Event.State >>= oULabel) &&
 		    !oULabel.matchAsciiL ("private:", 8, 0) &&
-		    !oULabel.matchAsciiL (".uno:", 5, 0))
+		    !oULabel.matchAsciiL (".uno:", 5, 0)    &&
+		    !oULabel.matchAsciiL ("slot:", 5, 0)    &&
+		    !oULabel.matchAsciiL ("service:", 8, 0) &&
+		    !oULabel.matchAsciiL (".cmd:", 5, 0)    &&
+		    !oULabel.matchAsciiL ("macro:///", 5, 0))
 		{	
 			oULabel = oULabel.replace ((sal_Unicode)0x007e, (sal_Unicode)0x005f);
 			gchar* label = g_utf16_to_utf8 (oULabel.getStr(),
@@ -270,8 +351,8 @@ class MenuItemStatusListener : public cppu::WeakImplHelper1 < XStatusListener >
 };
 
 FrameHelper::FrameHelper(const Reference< XMultiServiceFactory >&  rServiceManager,
-                    const Reference< XFrame >&        xFrame,
-                    DbusmenuServer*                   server)
+                         const Reference< XFrame >&        xFrame,
+                         DbusmenuServer*                   server)
 {
 	xMSF = rServiceManager;
 	this->xFrame = xFrame;
@@ -280,12 +361,11 @@ FrameHelper::FrameHelper(const Reference< XMultiServiceFactory >&  rServiceManag
 	//Get xUICommands database (to retrieve labels, see FrameJob::getLabelFromCommandURL ())                                          
 	Reference < XNameAccess > xNameAccess (xMSF->createInstance(OUString::createFromAscii("com.sun.star.frame.UICommandDescription")),
 	                                       UNO_QUERY);
-	Reference < XModuleManager > xModuleManager(xMSF->createInstance(OUString::createFromAscii("com.sun.star.frame.ModuleManager")),
-	                                            UNO_QUERY);
-	xNameAccess->getByName(xModuleManager->identify(xFrame)) >>= xUICommands;
+   	xMM = Reference < XModuleManager> (xMSF->createInstance(OUString::createFromAscii("com.sun.star.frame.ModuleManager")),
+	                                   UNO_QUERY);
+	xNameAccess->getByName(xMM->identify(xFrame)) >>= xUICommands;
 	
-	
-	xdp = Reference < XDispatchProvider > (xFrame, UNO_QUERY_THROW);
+	xdp = Reference < XDispatchProvider > (xFrame, UNO_QUERY);
     xTrans = Reference < XURLTransformer > (xMSF->createInstance( rtl::OUString::createFromAscii("com.sun.star.util.URLTransformer" )), UNO_QUERY);
 	
 	xSL = (XStatusListener*)new MenuItemStatusListener (this);
@@ -294,9 +374,10 @@ FrameHelper::FrameHelper(const Reference< XMultiServiceFactory >&  rServiceManag
 		                                  g_free,
 		                                  destroy_menu_item_info);
 		                                  
-		                                  
-   	xMM = Reference < XModuleManager> (xMSF->createInstance(OUString::createFromAscii("com.sun.star.frame.ModuleManager")),
-	                                   UNO_QUERY);
+	// This initializes the shortcut database
+	getAcceleratorConfigurations (xFrame->getController()->getModel (), xMM);
+
+	// This information is needed for the dynamic submenus
 	xPCF = Reference < XMultiComponentFactory > (xMSF->createInstance(OUString::createFromAscii("com.sun.star.frame.PopupMenuControllerFactory")),
 	                                             UNO_QUERY);
 	
@@ -317,13 +398,14 @@ FrameHelper::FrameHelper(const Reference< XMultiServiceFactory >&  rServiceManag
 	watcher_set = FALSE;
 }
 
+void SAL_CALL 
+FrameHelper::disposing (const EventObject& aEvent) throw (RuntimeException)
+{}
+
 FrameHelper::~FrameHelper()
 {
 	if (server)
 		g_object_unref (server);
-		
-	if (root)
-		g_object_unref (root);
 
 	if (watcher_set)
 		g_bus_unwatch_name (watcher);
@@ -372,7 +454,7 @@ FrameHelper::getCommandsInfo ()
 unsigned long
 FrameHelper::getXID ()
 {
-	Reference< XSystemDependentWindowPeer > xWin( xFrame->getContainerWindow(), UNO_QUERY_THROW );
+	Reference< XSystemDependentWindowPeer > xWin( xFrame->getContainerWindow(), UNO_QUERY );
 
 	if (!xWin.is())
 		return 0;
@@ -391,17 +473,70 @@ void
 FrameHelper::dispatchCommand (OUString command)
 {
 	Reference < XDispatchHelper > rDispatchHelper (xMSF->createInstance(OUString(RTL_CONSTASCII_USTRINGPARAM("com.sun.star.frame.DispatchHelper"))),
-	                                               UNO_QUERY_THROW);
+	                                               UNO_QUERY);
 	Reference < XDispatchHelper > xdh(rDispatchHelper);
 	
 	
-	xdh->executeDispatch (Reference < XDispatchProvider > (xFrame, UNO_QUERY_THROW),
+	xdh->executeDispatch (Reference < XDispatchProvider > (xFrame, UNO_QUERY),
 	                      command,
 	                      OUString::createFromAscii(""),
 	                      0,
 	                      Sequence < PropertyValue > ());
 }
 
+
+void SAL_CALL
+FrameHelper::frameAction(const FrameActionEvent& action) throw (RuntimeException)
+{
+	//We deregister this action listener, it is then disposed and the dbusmenuserver shutdown	
+	if (action.Action == frame::FrameAction_COMPONENT_DETACHING)
+	{
+		GError *error = NULL;
+		xFrame->removeFrameActionListener (this);
+		Reference< XPropertySet > frameProps (xFrame, UNO_QUERY);
+		Reference < XLayoutManager > xLayoutManager(frameProps->getPropertyValue(OUString::createFromAscii("LayoutManager")),
+		                                            UNO_QUERY);
+		xLayoutManager->showElement (OUString::createFromAscii("private:resource/menubar/menubar"));
+		
+		unsigned long xid = getXID();
+	
+ 		GDBusProxy *proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+		                                                   G_DBUS_PROXY_FLAGS_NONE,
+		                                                   NULL,
+		                                                   "com.canonical.AppMenu.Registrar",
+		                                                   "/com/canonical/AppMenu/Registrar",
+		                                                   "com.canonical.AppMenu.Registrar",
+ 		                                                   NULL,
+ 		                                                   &error);
+		if (error)
+		{
+			g_warning ("Couldn't get /com/canonical/AppMenu/Registrar proxy");
+			g_error_free (error);
+			return;
+		}
+
+
+		//TODO: Check if window is registered already
+		g_dbus_proxy_call_sync (proxy,
+			                    "UnregisterWindow",
+			                    g_variant_new ("(u)", (guint32)xid),
+			                    G_DBUS_CALL_FLAGS_NONE,
+			                    -1,
+			                    NULL,
+			                    &error);
+	
+		if (error)
+		{
+			g_warning ("Couldn't call /com/canonical/AppMenu/Registrar.UnregisterWindow");
+			g_error_free (error);
+		}
+		
+		return;
+	}
+}
+
+
+
 void
 FrameHelper::rebuildMenu (Reference < XMenu >  xMenu,
                           DbusmenuMenuitem    *parent)
@@ -436,8 +571,6 @@ FrameHelper::rebuildMenu (Reference < XMenu >  xMenu,
 	
 	for (guint16 i = 0; i<count; i++)
 	{
-		gchar * label = NULL;
-		gint isEnabled = 0;
 		Reference < XMenuExtended > xMenuEx (xMenu, UNO_QUERY);
 		guint16 id = xMenu->getItemId (i);
 		OUString oUCommand = xMenuEx->getCommand (id);
@@ -472,6 +605,13 @@ FrameHelper::rebuildMenu (Reference < XMenu >  xMenu,
 		OString command = OUStringToOString (oUCommand, RTL_TEXTENCODING_ASCII_US);
 		dbusmenu_menuitem_property_set (item, "CommandURL", command.getStr());
 
+/*		Sequence < KeyEvent > evs = findShortcutForCommand (oUCommand);
+		for (int j = 0; j < evs.getLength (); j++)
+		{
+			//if KeyCode is 0 continue until we get a valid keycode
+			g_debug ("%s: %x - %x - %x - %x", command.getStr(), evs[i].KeyCode, evs[i].KeyChar, evs[i].KeyFunc, evs[i].Modifiers);
+		}*/
+
 		MenuItemInfo* commInfo = (MenuItemInfo*)g_hash_table_lookup (commandsInfo, (gconstpointer)command.getStr());
 		if (!commInfo)
 		{
@@ -498,7 +638,7 @@ FrameHelper::rebuildMenu (Reference < XMenu >  xMenu,
 		}
 
 		//Update the check state directly from the data, this is more reliable
-		
+
 		Reference < XPopupMenu > popUp (xMenu, UNO_QUERY);
 		if (popUp.is() && popUp->isItemChecked (id))
 		{
@@ -566,7 +706,7 @@ FrameHelper::rebuildMenu (Reference < XMenu >  xMenu,
 			g_list_free_full (dbusmenu_menuitem_take_children (item), destroy_menuitem);
 		}		
 	}
-	
+
 	return;
 }
 
@@ -623,7 +763,7 @@ FrameHelper::rebuildMenuFromRoot ()
 gboolean
 FrameHelper::isSpecialSubmenu (OUString command)
 {
-	const gchar * specialSubmenus[11] = { ".uno:CharFontName",
+	const gchar * specialSubmenus[11] = {".uno:CharFontName",
 	                                     ".uno:FontHeight",
 	                                     ".uno:ObjectMenue",
 	                                     ".uno:InsertPageHeader",
@@ -642,3 +782,59 @@ FrameHelper::isSpecialSubmenu (OUString command)
 	}
 	return FALSE;
 }
+
+//Set all the accelerator configuration sources
+void
+FrameHelper::getAcceleratorConfigurations (Reference < XModel >        xModel,
+                                           Reference < XModuleManager> xModuleManager)
+{
+	//Get document shortcut database
+	Reference< XUIConfigurationManagerSupplier > docUISupplier(xModel, UNO_QUERY);
+    Reference< XUIConfigurationManager >         docUIManager = docUISupplier->getUIConfigurationManager();
+    Reference< XAcceleratorConfiguration >       docAccelConf(docUIManager->getShortCutManager(), UNO_QUERY);
+   	this->docAccelConf = docAccelConf;
+
+	//Get module shurtcut database
+    Reference< XModuleUIConfigurationManagerSupplier > modUISupplier(xMSF->createInstance(OUString::createFromAscii("com.sun.star.ui.ModuleUIConfigurationManagerSupplier")),
+	                                                                 UNO_QUERY);
+	Reference< XUIConfigurationManager >   modUIManager = modUISupplier->getUIConfigurationManager(xModuleManager->identify(xFrame));
+	Reference< XAcceleratorConfiguration > modAccelConf(modUIManager->getShortCutManager(), UNO_QUERY);
+	this->modAccelConf = modAccelConf;
+	
+	//Get global shortcut database
+	Reference< XAcceleratorConfiguration > globAccelConf(xMSF->createInstance(OUString::createFromAscii("com.sun.star.ui.GlobalAcceleratorConfiguration")),
+	                                                     UNO_QUERY);
+	this->globAccelConf = globAccelConf;
+}
+
+Sequence < KeyEvent >
+FrameHelper::findShortcutForCommand (OUString command)
+
+{
+	Sequence < KeyEvent > events (0);
+
+	try
+	{
+		events = docAccelConf->getKeyEventsByCommand (command);
+	}
+	catch(...)
+	{
+		try
+		{
+			events = modAccelConf->getKeyEventsByCommand (command);
+		}
+		catch(...)
+		{
+			try 
+			{
+				events = globAccelConf->getKeyEventsByCommand (command);
+			}
+			catch(...)
+			{
+				return events;
+			}
+		}
+	}
+
+	return events;
+}
