@@ -21,6 +21,7 @@
  *
  */
 
+#include <com/sun/star/awt/KeyEvent.hpp>
 #include <com/sun/star/awt/XMenu.hpp>
 #include <com/sun/star/beans/PropertyValue.hpp>
 #include <com/sun/star/frame/XFrame.hpp>
@@ -32,14 +33,17 @@
 #include <com/sun/star/frame/FrameAction.hpp>
 #include <com/sun/star/frame/XDispatchProvider.hpp>
 #include <com/sun/star/frame/XModuleManager.hpp>
+#include <com/sun/star/frame/XModel.hpp>
 #include <cppuhelper/implbase1.hxx>
 #include <com/sun/star/lang/EventObject.hpp>
 #include <com/sun/star/container/XNameAccess.hpp>
 #include <com/sun/star/util/XURLTransformer.hpp>
+#include <com/sun/star/ui/XAcceleratorConfiguration.hpp>
 
 #include <libdbusmenu-glib/server.h>
 #include <libdbusmenu-glib/menuitem.h>
 
+using com::sun::star::awt::KeyEvent;
 using com::sun::star::awt::XMenu;
 using com::sun::star::beans::PropertyValue;
 using com::sun::star::container::XNameAccess;
@@ -50,6 +54,7 @@ using com::sun::star::frame::XFrameActionListener;
 using com::sun::star::frame::XStatusListener;
 using com::sun::star::frame::XDispatchProvider;
 using com::sun::star::frame::XModuleManager;
+using com::sun::star::frame::XModel;
 using com::sun::star::lang::EventObject;
 using com::sun::star::lang::XMultiServiceFactory;
 using com::sun::star::lang::XMultiComponentFactory;
@@ -58,6 +63,7 @@ using com::sun::star::uno::Reference;
 using com::sun::star::uno::Sequence;
 using com::sun::star::uno::RuntimeException;
 using com::sun::star::util::XURLTransformer;
+using com::sun::star::ui::XAcceleratorConfiguration;
 using rtl::OUString;
 
 /* This class is a helper in charge of closing the dbusmenu server when a frame is closed,
@@ -85,18 +91,28 @@ class FrameHelper : public cppu::WeakImplHelper1 < XFrameActionListener >
 	Sequence  < Any >                    args;
 	
 	gboolean                             isSpecialSubmenu (OUString command);
+	
+	//This is to build the shortcut database
+    Reference< XAcceleratorConfiguration > docAccelConf;
+    Reference< XAcceleratorConfiguration > modAccelConf;
+    Reference< XAcceleratorConfiguration > globAccelConf;
+
+	void getAcceleratorConfigurations (Reference < XModel >,
+	                                   Reference < XModuleManager>);
+
+	Sequence < KeyEvent > findShortcutForCommand  (OUString);
 
   public:
-	FrameHelper(const Reference< XMultiServiceFactory >&  rServiceManager,
-	                    const Reference< XFrame >&        xFrame,
-	                    DbusmenuServer*                   server);
+	FrameHelper(const Reference< XMultiServiceFactory >&,
+	            const Reference< XFrame >&,
+	            DbusmenuServer*);
 
 	virtual ~FrameHelper();
 	virtual void SAL_CALL frameAction(const FrameActionEvent& action)
-	  throw (RuntimeException) {}
+	  throw (RuntimeException);
 	
 	virtual void SAL_CALL disposing(const EventObject& aEvent)
-	  throw (RuntimeException) {}
+	  throw (RuntimeException);
 	
 	//Setters
 	void setRootItem         (DbusmenuMenuitem *);
